@@ -1,4 +1,4 @@
-# $Id: test.pl,v 1.6 2001/05/21 03:49:49 jesse Exp $
+# $Id: test.pl,v 1.8 2001/06/25 03:01:19 jesse Exp $
 # Before `make install' is performed this script should be runnable with
 # `make test'. After `make install' it should work as `perl test.pl'
 
@@ -7,7 +7,7 @@
 # Change 1..1 below to 1..last_test_to_print .
 # (It may become useful if the test is moved to ./t subdirectory.)
 
-BEGIN { $| = 1; print "1..11\n"; }
+BEGIN { $| = 1; print "1..16\n"; }
 END {print "not ok 1\n" unless $loaded;}
 use CGI::Application;
 $loaded = 1;
@@ -23,6 +23,8 @@ print "ok 1\n";
 use lib './test';
 use TestApp;
 use TestApp2;
+use TestApp3;
+use TestApp4;
 
 $ENV{CGI_APP_RETURN_ONLY} = 1;
 
@@ -130,4 +132,62 @@ if (defined($t11_cgiapp_init_state) && ($t11_cgiapp_init_state eq 'true')) {
 
 
 
+# Test 12: Test to make sure mode_param() can contain subref
+my $t12_ta_obj = TestApp3->new();
+$t12_ta_obj->query(CGI->new({'go_to_mode' => 'subref_modeparam'}));
+my $t12_output = $t12_ta_obj->run();
+if (($t12_output =~ /^Content\-Type\:\ text\/html/) && ($t12_output =~ /Hello\ World\:\ subref\_modeparam\ OK/)) {
+	print "ok 12\n";
+} else {
+	print "not ok 12\n";
+}
+
+
+
+# Test 13: Test to make sure that "false" run-modes are valid -- won't default to start_mode()
+my $t13_ta_obj = TestApp3->new();
+$t13_ta_obj->query(CGI->new({'go_to_mode' => ''}));
+my $t13_output = $t13_ta_obj->run();
+if (($t13_output =~ /^Content\-Type\:\ text\/html/) && ($t13_output =~ /Hello\ World\:\ blank\_mode\ OK/)) {
+	print "ok 13\n";
+} else {
+	print "not ok 13\n";
+}
+
+
+
+# Test 14: Test to make sure that undef run-modes will default to start_mode()
+my $t14_ta_obj = TestApp3->new();
+$t14_ta_obj->query(CGI->new({'go_to_mode' => 'undef_rm'}));
+my $t14_output = $t14_ta_obj->run();
+if (($t14_output =~ /^Content\-Type\:\ text\/html/) && ($t14_output =~ /Hello\ World\:\ default\_mode\ OK/)) {
+	print "ok 14\n";
+} else {
+	print "not ok 14\n";
+}
+
+
+
+# Test 15: Test run-modes returning scalar-refs instead of scalars
+my $t15_ta_obj = TestApp4->new();
+my $t15_output = $t15_ta_obj->run();
+if (($t15_output =~ /^Content\-Type\:\ text\/html/) && ($t15_output =~ /Hello\ World\:\ subref\_test\ OK/)) {
+	print "ok 15\n";
+} else {
+	print "not ok 15\n";
+}
+
+
+# Test 16: Test "AUTOLOAD" run-mode
+my $t16_ta_obj = TestApp4->new();
+$t16_ta_obj->query(CGI->new({'rm' => 'undefined_mode'}));
+my $t16_output = $t16_ta_obj->run();
+if (($t16_output =~ /^Content\-Type\:\ text\/html/) && ($t16_output =~ /Hello\ World\:\ undefined\_mode\ OK/)) {
+	print "ok 16\n";
+} else {
+	print "not ok 16\n";
+}
+
+
+
 # All done!
